@@ -1,6 +1,3 @@
-"""
-.. include:: ../README.md
-"""
 from aligned.compiler.feature_factory import (
     UUID,
     Bool,
@@ -9,10 +6,11 @@ from aligned.compiler.feature_factory import (
     Float,
     Int32,
     Int64,
+    Json,
     String,
     Timestamp,
 )
-from aligned.compiler.model import Model
+from aligned.compiler.model import ModelContract
 from aligned.data_source.stream_data_source import HttpStreamSource
 from aligned.feature_store import FeatureStore
 from aligned.feature_view import (
@@ -20,17 +18,21 @@ from aligned.feature_view import (
     CombinedFeatureViewMetadata,
     FeatureView,
     FeatureViewMetadata,
+    feature_view,
 )
-from aligned.local.source import FileSource
-from aligned.psql.data_source import PostgreSQLConfig
-from aligned.redis.config import RedisConfig
-from aligned.redshift.data_source import RedshiftSQLConfig
-from aligned.s3.config import AwsS3Config
+from aligned.schemas.text_vectoriser import TextVectoriserModel
+from aligned.sources.kafka import KafkaConfig
+from aligned.sources.local import FileSource
+from aligned.sources.psql import PostgreSQLConfig
+from aligned.sources.redis import RedisConfig
+from aligned.sources.redshift import RedshiftSQLConfig
+from aligned.sources.s3 import AwsS3Config
 
 __all__ = [
     'FeatureStore',
     'FeatureView',
     'FeatureViewMetadata',
+    'feature_view',
     'CombinedFeatureView',
     'CombinedFeatureViewMetadata',
     # Batch Data sources
@@ -42,6 +44,8 @@ __all__ = [
     'HttpStreamSource',
     # Online Source
     'RedisConfig',
+    # Streaming Sources
+    'KafkaConfig',
     # Types
     'Entity',
     'String',
@@ -53,5 +57,7 @@ __all__ = [
     'Float',
     'EventTimestamp',
     'Timestamp',
-    'Model',
+    'Json',
+    'ModelContract',
+    'TextVectoriserModel',
 ]
