@@ -8,24 +8,36 @@ import pandas as pd
 import polars as pl
 
 from aligned.compiler.constraint_factory import ConstraintFactory, LiteralFactory
+from aligned.compiler.vector_index_factory import VectorIndexFactory
 from aligned.data_source.stream_data_source import StreamDataSource
 from aligned.schemas.constraints import (
     Constraint,
+    EndsWith,
     InDomain,
     LowerBound,
     LowerBoundInclusive,
     MaxLength,
     MinLength,
+    Unique,
+    Regex,
+    StartsWith,
     UpperBound,
     UpperBoundInclusive,
 )
 from aligned.schemas.derivied_feature import DerivedFeature
+from aligned.schemas.event_trigger import EventTrigger as EventTriggerSchema
 from aligned.schemas.feature import EventTimestamp as EventTimestampFeature
 from aligned.schemas.feature import Feature, FeatureLocation, FeatureReferance, FeatureType
+from aligned.schemas.literal_value import LiteralValue
+from aligned.schemas.target import ClassificationTarget as ClassificationTargetSchemas
+from aligned.schemas.target import ClassTargetProbability
+from aligned.schemas.target import RegressionTarget as RegressionTargetSchemas
 from aligned.schemas.transformation import TextVectoriserModel, Transformation
+from aligned.schemas.vector_storage import VectorStorage
 
 if TYPE_CHECKING:
     from aligned.compiler.transformation_factory import FillNaStrategy
+    from aligned.sources.s3 import AwsS3Config
 
 
 class TransformationFactory:
@@ -74,7 +86,7 @@ class EventTrigger:
 @dataclass
 class TargetProbability:
     of_value: Any
-    target: Target
+    target: ClassificationLabel
     _name: str | None = None
 
     def __hash__(self) -> int:
@@ -83,6 +95,12 @@ class TargetProbability:
     def __set_name__(self, owner, name):
         self._name = name
 
+    def compile(self) -> ClassTargetProbability:
+        return ClassTargetProbability(
+            outcome=LiteralValue.from_value(self.of_value),
+            feature=Feature(self._name, dtype=FeatureType('').float),
+        )
+
 
 class FeatureReferencable:
     def feature_referance(self) -> FeatureReferance:
@@ -90,7 +108,7 @@ class FeatureReferencable:
 
 
 @dataclass
-class Target(FeatureReferencable):
+class RegressionLabel(FeatureReferencable):
     feature: FeatureFactory
     event_trigger: EventTrigger | None = field(default=None)
     ground_truth_event: StreamDataSource | None = field(default=None)
@@ -107,19 +125,91 @@ class Target(FeatureReferencable):
             raise ValueError('Missing location, can not create reference')
         return FeatureReferance(self._name, self._location, self.feature.dtype)
 
-    def listen_to_ground_truth_event(self, stream: StreamDataSource) -> Target:
-        return Target(
+    def listen_to_ground_truth_event(self, stream: StreamDataSource) -> RegressionLabel:
+        return RegressionLabel(
             feature=self.feature,
             event_trigger=self.event_trigger,
             ground_truth_event=stream,
         )
 
-    def send_ground_truth_event(self, when: Bool, sink_to: StreamDataSource) -> Target:
+    def send_ground_truth_event(self, when: Bool, sink_to: StreamDataSource) -> RegressionLabel:
         assert when.dtype == FeatureType('').bool, 'A trigger needs a boolean condition'
 
-        return Target(self.feature, EventTrigger(when, sink_to))
+        return RegressionLabel(
+            self.feature, EventTrigger(when, sink_to), ground_truth_event=self.ground_truth_event
+        )
+
+    def compile(self) -> RegressionTargetSchemas:
+        on_ground_truth_event = self.ground_truth_event
+        trigger = self.event_trigger
+
+        if self.event_trigger:
+            event = self.event_trigger
+            if not event.condition._name:
+                event.condition._name = '0'
+
+            trigger = EventTriggerSchema(
+                event.condition.compile(), event=event.event, payload={self.feature.feature()}
+            )
+            if not on_ground_truth_event:
+                on_ground_truth_event = event.event
+
+        return RegressionTargetSchemas(
+            self.feature.feature_referance(),
+            feature=Feature(self._name, self.feature.dtype),
+            on_ground_truth_event=on_ground_truth_event,
+            event_trigger=trigger,
+        )
+
+
+@dataclass
+class ClassificationLabel(FeatureReferencable):
+    feature: FeatureFactory
+    event_trigger: EventTrigger | None = field(default=None)
+    ground_truth_event: StreamDataSource | None = field(default=None)
+    _name: str | None = field(default=None)
+    _location: FeatureLocation | None = field(default=None)
+
+    def __set_name__(self, owner, name):
+        self._name = name
+
+    def feature_referance(self) -> FeatureReferance:
+        if not self._name:
+            raise ValueError('Missing name, can not create reference')
+        if not self._location:
+            raise ValueError('Missing location, can not create reference')
+        return FeatureReferance(self._name, self._location, self.feature.dtype)
+
+    def listen_to_ground_truth_event(self, stream: StreamDataSource) -> ClassificationLabel:
+        return ClassificationLabel(
+            feature=self.feature,
+            event_trigger=self.event_trigger,
+            ground_truth_event=stream,
+        )
+
+    def send_ground_truth_event(self, when: Bool, sink_to: StreamDataSource) -> ClassificationLabel:
+        assert when.dtype == FeatureType('').bool, 'A trigger needs a boolean condition'
+
+        return ClassificationLabel(self.feature, EventTrigger(when, sink_to))
 
     def probability_of(self, value: Any) -> TargetProbability:
+        """Define a value that will be the probability of a certain target class.
+
+        This is mainly intended to be used for classification problems with low cardinality.
+
+        For example, if the target is a binary classification, then the probability of the target.
+        being 1 can be defined by:
+
+        >>> target.probability_of(1)
+
+        For cases where the target have a high cardinality can `probabilities` be used instead.
+
+        Args:
+            value (Any): The class that will be the probability of the target.
+
+        Returns:
+            TargetProbability: A feature that contains the probability of the target class.
+        """
 
         if not isinstance(value, self.feature.dtype.python_type):
             raise ValueError(
@@ -131,6 +221,28 @@ class Target(FeatureReferencable):
 
         return TargetProbability(value, self)
 
+    def compile(self) -> ClassificationTargetSchemas:
+        on_ground_truth_event = self.ground_truth_event
+        trigger = self.event_trigger
+
+        if self.event_trigger:
+            event = self.event_trigger
+            if not event.condition._name:
+                event.condition._name = '0'
+
+            trigger = EventTriggerSchema(
+                event.condition.compile(), event=event.event, payload={self.feature.feature()}
+            )
+            if not on_ground_truth_event:
+                on_ground_truth_event = event.event
+
+        return ClassificationTargetSchemas(
+            self.feature.feature_referance(),
+            feature=Feature(self._name, self.feature.dtype),
+            on_ground_truth_event=on_ground_truth_event,
+            event_trigger=trigger,
+        )
+
 
 class FeatureFactory(FeatureReferencable):
     """
@@ -188,13 +300,16 @@ class FeatureFactory(FeatureReferencable):
             constraints=self.constraints,
         )
 
-    def as_target(self) -> Target:
-        return Target(self)
+    def as_classification_label(self) -> ClassificationLabel:
+        return ClassificationLabel(self)
+
+    def as_regression_label(self) -> RegressionLabel:
+        return RegressionLabel(self)
 
     def compile(self) -> DerivedFeature:
 
         if not self.transformation:
-            raise ValueError('Trying to create a derived feature with no transformation')
+            raise ValueError(f'Trying to create a derived feature with no transformation, {self.name}')
 
         return DerivedFeature(
             name=self.name,
@@ -204,7 +319,7 @@ class FeatureFactory(FeatureReferencable):
             depth=self.depth(),
             description=self._description,
             tags=None,
-            constraints=None,
+            constraints=self.constraints,
         )
 
     def depth(self) -> int:
@@ -321,6 +436,14 @@ class FeatureFactory(FeatureReferencable):
         return instance
 
 
+class CouldBeModelVersion:
+    def as_model_Version(self) -> ModelVersion:
+        if isinstance(self, FeatureFactory):
+            return ModelVersion(self)
+
+        raise ValueError(f'{self} is not a feature factory, and can therefore not be a model version')
+
+
 class CouldBeEntityFeature:
     def as_entity(self) -> Entity:
         if isinstance(self, FeatureFactory):
@@ -424,11 +547,14 @@ class ArithmeticFeature(ComparableFeature):
         feature.transformation = AdditionBetweenFactory(self, other)
         return feature
 
-    def __truediv__(self, other: FeatureFactory) -> Float:
+    def __truediv__(self, other: FeatureFactory | Any) -> Float:
         from aligned.compiler.transformation_factory import RatioFactory
 
         feature = Float()
-        feature.transformation = RatioFactory(self, other)
+        if isinstance(other, FeatureFactory):
+            feature.transformation = RatioFactory(self, other)
+        else:
+            feature.transformation = RatioFactory(self, LiteralValue.from_value(other))
         return feature
 
     def __floordiv__(self, other: FeatureFactory) -> Float:
@@ -445,6 +571,26 @@ class ArithmeticFeature(ComparableFeature):
         feature.transformation = AbsoluteFactory(self)
         return feature
 
+    def __mul__(self, other: FeatureFactory | Any) -> Float:
+        from aligned.compiler.transformation_factory import MultiplyFactory
+
+        feature = Float()
+        if isinstance(other, FeatureFactory):
+            feature.transformation = MultiplyFactory(self, other)
+        else:
+            feature.transformation = MultiplyFactory(self, LiteralValue.from_value(other))
+        return feature
+
+    def __rmul__(self, other: FeatureFactory | Any) -> Float:
+        from aligned.compiler.transformation_factory import MultiplyFactory
+
+        feature = Float()
+        if isinstance(other, FeatureFactory):
+            feature.transformation = MultiplyFactory(self, other)
+        else:
+            feature.transformation = MultiplyFactory(self, LiteralValue.from_value(other))
+        return feature
+
     def __pow__(self, other: FeatureFactory | Any) -> Float:
         from aligned.compiler.transformation_factory import PowerFactory
 
@@ -459,6 +605,13 @@ class ArithmeticFeature(ComparableFeature):
         feature.transformation = LogTransformFactory(self)
         return feature
 
+    def clip(self: T, lower_bound: float, upper_bound: float) -> T:
+        from aligned.compiler.transformation_factory import ClipFactory
+
+        feature = Float()
+        feature.transformation = ClipFactory(self, lower_bound, upper_bound)
+        return feature
+
 
 class DecimalOperations(FeatureFactory):
     def __round__(self) -> Int64:
@@ -468,6 +621,9 @@ class DecimalOperations(FeatureFactory):
         feature.transformation = RoundFactory(self)
         return feature
 
+    def round(self) -> Int64:
+        return self.__round__()
+
     def __ceil__(self) -> Int64:
         from aligned.compiler.transformation_factory import CeilFactory
 
@@ -475,6 +631,9 @@ class DecimalOperations(FeatureFactory):
         feature.transformation = CeilFactory(self)
         return feature
 
+    def cail(self) -> Int64:
+        return self.__ceil__()
+
     def __floor__(self) -> Int64:
         from aligned.compiler.transformation_factory import FloorFactory
 
@@ -482,6 +641,9 @@ class DecimalOperations(FeatureFactory):
         feature.transformation = FloorFactory(self)
         return feature
 
+    def floor(self) -> Int64:
+        return self.__floor__()
+
 
 class TruncatableFeature(FeatureFactory):
     def __trunc__(self: T) -> T:
@@ -558,6 +720,36 @@ class DateFeature(FeatureFactory):
         feature.transformation = DateComponentFactory(component, self)
         return feature
 
+    def day(self) -> Int32:
+        return self.date_component('day')
+
+    def hour(self) -> Int32:
+        return self.date_component('hour')
+
+    def second(self) -> Int32:
+        return self.date_component('second')
+
+    def minute(self) -> Int32:
+        return self.date_component('minute')
+
+    def quarter(self) -> Int32:
+        return self.date_component('quarter')
+
+    def week(self) -> Int32:
+        return self.date_component('week')
+
+    def year(self) -> Int32:
+        return self.date_component('year')
+
+    def month(self) -> Int32:
+        return self.date_component('month')
+
+    def weekday(self) -> Int32:
+        return self.date_component('dayofweek')
+
+    def day_of_year(self) -> Int32:
+        return self.date_component('ordinal_day')
+
 
 class Bool(EquatableFeature, LogicalOperatableFeature):
     @property
@@ -580,7 +772,7 @@ class Float(ArithmeticFeature, DecimalOperations):
         return ArithmeticAggregation(self)
 
 
-class Int32(ArithmeticFeature, CouldBeEntityFeature):
+class Int32(ArithmeticFeature, CouldBeEntityFeature, CouldBeModelVersion):
     def copy_type(self) -> Int32:
         return Int32()
 
@@ -592,7 +784,7 @@ class Int32(ArithmeticFeature, CouldBeEntityFeature):
         return ArithmeticAggregation(self)
 
 
-class Int64(ArithmeticFeature, CouldBeEntityFeature):
+class Int64(ArithmeticFeature, CouldBeEntityFeature, CouldBeModelVersion):
     def copy_type(self) -> Int64:
         return Int64()
 
@@ -612,6 +804,15 @@ class UUID(FeatureFactory, CouldBeEntityFeature):
     def dtype(self) -> FeatureType:
         return FeatureType('').uuid
 
+    def aggregate(self) -> CategoricalAggregation:
+        return CategoricalAggregation(self)
+
+
+class UniqueValidateable(FeatureFactory):
+    def is_unique(self: T) -> T:
+        self._add_constraint(Unique())
+        return self
+
 
 class LengthValidatable(FeatureFactory):
     def min_length(self: T, length: int) -> T:
@@ -623,7 +824,28 @@ class LengthValidatable(FeatureFactory):
         return self
 
 
-class String(CategoricalEncodableFeature, NumberConvertableFeature, CouldBeEntityFeature, LengthValidatable):
+class StringValidatable(FeatureFactory):
+    def validate_regex(self: T, regex: str) -> T:
+        self._add_constraint(Regex(regex))
+        return self
+
+    def validate_endswith(self: T, suffix: str) -> T:
+        self._add_constraint(EndsWith(suffix))
+        return self
+
+    def validate_startswith(self: T, prefix: str) -> T:
+        self._add_constraint(StartsWith(prefix))
+        return self
+
+
+class String(
+    CategoricalEncodableFeature,
+    NumberConvertableFeature,
+    CouldBeModelVersion,
+    CouldBeEntityFeature,
+    LengthValidatable,
+    StringValidatable,
+):
     def copy_type(self) -> String:
         return String()
 
@@ -656,15 +878,76 @@ class String(CategoricalEncodableFeature, NumberConvertableFeature, CouldBeEntit
 
         feature = Embedding()
         feature.transformation = WordVectoriserFactory(self, model)
+        feature.embedding_size = model.embedding_size
         return feature
 
-    def append(self, feature: FeatureFactory | str) -> String:
+    def embedding(self, model: TextVectoriserModel) -> Embedding:
+        return self.sentence_vector(model)
+
+    def append(self, other: FeatureFactory | str) -> String:
         from aligned.compiler.transformation_factory import AppendStrings
 
         feature = String()
-        feature.transformation = AppendStrings(self, feature)
+        if isinstance(other, FeatureFactory):
+            feature.transformation = AppendStrings(self, other)
+        else:
+            feature.transformation = AppendStrings(self, LiteralValue.from_value(other))
         return feature
 
+    def prepend(self, other: FeatureFactory | str) -> String:
+        from aligned.compiler.transformation_factory import AppendStrings, PrependConstString
+
+        feature = String()
+        if isinstance(other, FeatureFactory):
+            feature.transformation = AppendStrings(other, self)
+        else:
+            feature.transformation = PrependConstString(other, self)
+        return feature
+
+    def as_presigned_aws_url(self, credentials: AwsS3Config, max_age_seconds: int | None = None) -> ImageUrl:
+        from aligned.compiler.transformation_factory import PresignedAwsUrlFactory
+
+        feature = ImageUrl()
+        feature.transformation = PresignedAwsUrlFactory(credentials, self, max_age_seconds or 30)
+
+        return feature
+
+    def as_image_url(self) -> ImageUrl:
+        image_url = ImageUrl()
+        image_url.transformation = self.transformation
+        return image_url
+
+
+class Json(FeatureFactory):
+    def copy_type(self: Json) -> Json:
+        return super().copy_type()
+
+    @property
+    def dtype(self) -> FeatureType:
+        return FeatureType('').string
+
+    def json_path_value_at(self, path: str, as_type: T) -> T:
+        from aligned.compiler.transformation_factory import JsonPathFactory
+
+        feature = as_type.copy_type()
+        feature.transformation = JsonPathFactory(self, path)
+        return feature
+
+
+class ModelVersion(FeatureFactory):
+
+    _dtype: FeatureFactory
+
+    @property
+    def dtype(self) -> FeatureType:
+        return self._dtype.dtype
+
+    def __init__(self, dtype: FeatureFactory):
+        self._dtype = dtype
+
+    def aggregate(self) -> CategoricalAggregation:
+        return CategoricalAggregation(self)
+
 
 class Entity(FeatureFactory):
 
@@ -707,6 +990,8 @@ class EventTimestamp(DateFeature, ArithmeticFeature):
 class Embedding(FeatureFactory):
 
     sub_type: FeatureFactory
+    embedding_size: int | None = None
+    indexes: list[VectorIndexFactory] | None = None
 
     def copy_type(self) -> Embedding:
         return Embedding()
@@ -715,8 +1000,49 @@ class Embedding(FeatureFactory):
     def dtype(self) -> FeatureType:
         return FeatureType('').embedding
 
+    def indexed(
+        self,
+        storage: VectorStorage,
+        metadata: list[FeatureFactory] | None = None,
+        embedding_size: int | None = None,
+    ) -> Embedding:
+        if self.indexes is None:
+            self.indexes = []
 
-class ImageUrl(FeatureFactory):
+        if not self.embedding_size:
+            assert embedding_size, 'An embedding size is needed in order to create a vector index'
+
+        self.indexes.append(
+            VectorIndexFactory(
+                vector_dim=self.embedding_size or embedding_size,
+                metadata=metadata or [],
+                storage=storage,
+            )
+        )
+        return self
+
+
+@dataclass
+class List(FeatureFactory):
+
+    sub_type: FeatureFactory
+
+    def copy_type(self) -> List:
+        return List(self.sub_type.copy_type())
+
+    @property
+    def dtype(self) -> FeatureType:
+        return FeatureType('').array
+
+    def contains(self, value: Any) -> Bool:
+        from aligned.compiler.transformation_factory import ArrayContainsFactory
+
+        feature = Bool()
+        feature.transformation = ArrayContainsFactory(LiteralValue.from_value(value), self)
+        return feature
+
+
+class ImageUrl(StringValidatable):
     @property
     def dtype(self) -> FeatureType:
         return FeatureType('').string
@@ -779,6 +1105,13 @@ class StringAggregation:
         )
         return feature
 
+    def count(self) -> Int32:
+        from aligned.compiler.aggregation_factory import CountAggregationFactory
+
+        feat = Int32()
+        feat.transformation = CountAggregationFactory(self.feature, group_by=[], time_window=self.time_window)
+        return feat
+
 
 @dataclass
 class CategoricalAggregation:
@@ -786,8 +1119,17 @@ class CategoricalAggregation:
     feature: FeatureFactory
     time_window: timedelta | None = None
 
-    def over(self, time_window: timedelta) -> CategoricalAggregation:
-        self.time_window = time_window
+    def over(
+        self,
+        weeks: float | None = None,
+        days: float | None = None,
+        hours: float | None = None,
+        minutes: float | None = None,
+        seconds: float | None = None,
+    ) -> ArithmeticAggregation:
+        self.time_window = timedelta(
+            weeks=weeks or 0, days=days or 0, hours=hours or 0, minutes=minutes or 0, seconds=seconds or 0
+        )
         return self
 
     def count(self) -> Int64:
