@@ -6,6 +6,7 @@ from typing import Any, Callable
 import pandas as pd
 import polars as pl
 
+from aligned import AwsS3Config
 from aligned.compiler.feature_factory import FeatureFactory, Transformation, TransformationFactory
 from aligned.schemas.transformation import LiteralValue, TextVectoriserModel
 
@@ -53,15 +54,17 @@ class NotNullFactory(TransformationFactory):
 class RatioFactory(TransformationFactory):
 
     numerator: FeatureFactory
-    denumerator: FeatureFactory
+    denumerator: FeatureFactory | LiteralValue
 
     @property
     def using_features(self) -> list[FeatureFactory]:
         return [self.numerator, self.denumerator]
 
     def compile(self) -> Transformation:
-        from aligned.schemas.transformation import Ratio
+        from aligned.schemas.transformation import DivideDenumeratorValue, Ratio
 
+        if isinstance(self.denumerator, LiteralValue):
+            return DivideDenumeratorValue(self.numerator.name, self.denumerator)
         return Ratio(self.numerator.name, self.denumerator.name)
 
 
@@ -81,6 +84,22 @@ class OrdinalFactory(TransformationFactory):
         return Ordinal(self.feature.name, self.orders)
 
 
+@dataclass
+class ArrayContainsFactory(TransformationFactory):
+
+    value: LiteralValue
+    in_feature: FeatureFactory
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return [self.in_feature]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import ArrayContains
+
+        return ArrayContains(self.in_feature.name, self.value)
+
+
 @dataclass
 class ContainsFactory(TransformationFactory):
 
@@ -256,16 +275,22 @@ class DifferanceBetweenFactory(TransformationFactory):
 class AdditionBetweenFactory(TransformationFactory):
 
     first_feature: FeatureFactory
-    second_feature: FeatureFactory
+    second_feature: FeatureFactory | Any
 
     @property
     def using_features(self) -> list[FeatureFactory]:
-        return [self.first_feature, self.second_feature]
+        if isinstance(self.second_feature, FeatureFactory):
+            return [self.first_feature, self.second_feature]
+        else:
+            return [self.first_feature]
 
     def compile(self) -> Transformation:
-        from aligned.schemas.transformation import Addition
+        from aligned.schemas.transformation import Addition, AdditionValue
 
-        return Addition(self.first_feature.name, self.second_feature.name)
+        if isinstance(self.second_feature, FeatureFactory):
+            return Addition(self.first_feature.name, self.second_feature.name)
+        else:
+            return AdditionValue(self.first_feature.name, LiteralValue.from_value(self.second_feature))
 
 
 @dataclass
@@ -419,10 +444,6 @@ class FillNaStrategy:
     def compile(self) -> Any:
         pass
 
-    @staticmethod
-    def mean(limit: int | None = None) -> 'FillNaStrategy':
-        return MeanFillNaStrategy(limit)
-
 
 @dataclass
 class ConstantFillNaStrategy(FillNaStrategy):
@@ -432,16 +453,6 @@ class ConstantFillNaStrategy(FillNaStrategy):
         return self.value
 
 
-@dataclass
-class MeanFillNaStrategy(FillNaStrategy):
-
-    limit: int | None = field(default=None)
-
-    def compile(self) -> Any:
-        logging.info('The Mean Fill strategy will be deprecated.')
-        return 0
-
-
 @dataclass
 class FillMissingFactory(TransformationFactory):
 
@@ -534,6 +545,7 @@ class PandasTransformationFactory(TransformationFactory):
         return self._using_features
 
     def compile(self) -> Transformation:
+        import inspect
         import types
 
         import dill
@@ -543,12 +555,12 @@ class PandasTransformationFactory(TransformationFactory):
         if isinstance(self.method, types.LambdaType) and self.method.__name__ == '<lambda>':
             return PandasLambdaTransformation(
                 method=dill.dumps(self.method),
-                code=dill.source.getsource(self.method),
+                code=inspect.getsource(self.method).strip(),
                 dtype=self.dtype.dtype,
             )
         else:
             return PandasFunctionTransformation(
-                code=dill.source.getsource(self.method),
+                code=inspect.getsource(self.method),
                 function_name=dill.source.getname(self.method),
                 dtype=self.dtype.dtype,
             )
@@ -566,21 +578,24 @@ class PolarsTransformationFactory(TransformationFactory):
         return self._using_features
 
     def compile(self) -> Transformation:
+        import inspect
         import types
 
         import dill
 
         from aligned.schemas.transformation import PolarsFunctionTransformation, PolarsLambdaTransformation
 
-        code = dill.source.getsource(self.method)
-
         if isinstance(self.method, pl.Expr):
             code = str(self.method)
-            self.method = lambda df, alias: df.with_column(self.method.alias(alias))
+            return PolarsLambdaTransformation(
+                method=dill.dumps(self.method), code=code.strip(), dtype=self.dtype.dtype
+            )
+        else:
+            code = inspect.getsource(self.method)
 
         if isinstance(self.method, types.LambdaType) and self.method.__name__ == '<lambda>':
             return PolarsLambdaTransformation(
-                method=dill.dumps(self.method), code=code, dtype=self.dtype.dtype
+                method=dill.dumps(self.method), code=code.strip(), dtype=self.dtype.dtype
             )
         else:
             return PolarsFunctionTransformation(
@@ -674,6 +689,7 @@ class AppendStrings(TransformationFactory):
 
     first_feature: FeatureFactory
     second_feature: FeatureFactory | LiteralValue
+    separator: str = field(default='')
 
     @property
     def using_features(self) -> list[FeatureFactory]:
@@ -688,4 +704,97 @@ class AppendStrings(TransformationFactory):
         if isinstance(self.second_feature, LiteralValue):
             return AppendConstString(self.first_feature.name, self.second_feature.value)
         else:
-            return AppendStrings(self.first_feature.name, self.second_feature.name)
+            return AppendStrings(self.first_feature.name, self.second_feature.name, self.separator)
+
+
+@dataclass
+class JsonPathFactory(TransformationFactory):
+
+    json_feature: FeatureFactory
+    path: str
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return [self.json_feature]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import JsonPath
+
+        return JsonPath(self.json_feature.name, self.path)
+
+
+@dataclass
+class PresignedAwsUrlFactory(TransformationFactory):
+
+    aws_config: AwsS3Config
+    url_feature: FeatureFactory
+    max_age_seconds: int = field(default=30)
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return [self.url_feature]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import PresignedAwsUrl
+
+        return PresignedAwsUrl(self.aws_config, self.url_feature.name, self.max_age_seconds)
+
+
+@dataclass
+class PrependConstString(TransformationFactory):
+
+    first_feature: str
+    second_feature: FeatureFactory
+    separator: str = field(default='')
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return [self.second_feature]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import PrependConstString
+
+        return PrependConstString(self.first_feature, self.second_feature.name)
+
+
+@dataclass
+class ClipFactory(TransformationFactory):
+
+    feature: FeatureFactory
+    lower_bound: int | float
+    upper_bound: int | float
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return [self.feature]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import Clip
+
+        return Clip(
+            self.feature.name,
+            LiteralValue.from_value(self.lower_bound),
+            LiteralValue.from_value(self.upper_bound),
+        )
+
+
+@dataclass
+class MultiplyFactory(TransformationFactory):
+
+    first: FeatureFactory
+    behind: FeatureFactory | LiteralValue
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        if isinstance(self.behind, LiteralValue):
+            return [self.first]
+        else:
+            return [self.first, self.behind]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import Multiply, MultiplyValue
+
+        if isinstance(self.behind, LiteralValue):
+            return MultiplyValue(self.first.name, self.behind)
+        else:
+            return Multiply(self.first.name, self.behind.name)
