@@ -1,9 +1,10 @@
 from aligned.feature_view.combined_view import CombinedFeatureView, CombinedFeatureViewMetadata
-from aligned.feature_view.feature_view import FeatureView, FeatureViewMetadata
+from aligned.feature_view.feature_view import FeatureView, FeatureViewMetadata, feature_view
 
 __all__ = [
     'FeatureView',
     'FeatureViewMetadata',
     'CombinedFeatureView',
     'CombinedFeatureViewMetadata',
+    'feature_view',
 ]
