@@ -2,212 +2,22 @@ from __future__ import annotations
 
 import logging
 from dataclasses import dataclass, field
-from datetime import datetime
 
 import pandas as pd
 import polars as pl
 
-from aligned.psql.data_source import PostgreSQLConfig, PostgreSQLDataSource
 from aligned.psql.jobs import PostgreSqlJob
+from aligned.redshift.sql_job import SqlColumn, TableFetch
 from aligned.request.retrival_request import RequestResult, RetrivalRequest
-from aligned.retrival_job import DateRangeJob, FactualRetrivalJob, FullExtractJob, RetrivalJob
-from aligned.schemas.derivied_feature import AggregatedFeature, AggregateOver
+from aligned.retrival_job import FactualRetrivalJob, RetrivalJob
+from aligned.schemas.derivied_feature import AggregatedFeature, AggregateOver, DerivedFeature
 from aligned.schemas.feature import FeatureLocation, FeatureType
 from aligned.schemas.transformation import RedshiftTransformation
+from aligned.sources.redshift import RedshiftSQLConfig, RedshiftSQLDataSource
 
 logger = logging.getLogger(__name__)
 
 
-@dataclass
-class SQLQuery:
-    sql: str
-
-
-@dataclass
-class SqlColumn:
-    selection: str
-    alias: str
-
-    @property
-    def sql_select(self) -> str:
-        if self.selection == self.alias:
-            return f'{self.selection}'
-        return f'{self.selection} AS "{self.alias}"'
-
-    def __hash__(self) -> int:
-        return hash(self.sql_select)
-
-
-@dataclass
-class SqlJoin:
-    table: str
-    conditions: list[str]
-
-
-@dataclass
-class TableFetch:
-    name: str
-    id_column: str
-    table: str | TableFetch
-    columns: set[SqlColumn]
-    schema: str | None = field(default=None)
-    joins: list[str] = field(default_factory=list)
-    conditions: list[str] = field(default_factory=list)
-    group_by: list[str] = field(default_factory=list)
-    order_by: str | None = field(default=None)
-
-    def sql_query(self, distinct: str | None = None) -> str:
-        # Select the core features
-        wheres = ''
-        order_by = ''
-        group_by = ''
-        select = 'SELECT'
-
-        if self.conditions:
-            wheres = 'WHERE ' + ' AND '.join(self.conditions)
-
-        if self.order_by:
-            order_by = 'ORDER BY ' + self.order_by
-
-        if self.group_by:
-            group_by = 'GROUP BY ' + ', '.join(self.group_by)
-
-        table_columns = [col.sql_select for col in self.columns]
-
-        if isinstance(self.table, TableFetch):
-            from_sql = f'FROM ({self.table.sql_query()}) as entities'
-        else:
-            schema = f'{self.schema}.' if self.schema else ''
-            from_sql = f"""FROM entities
-        LEFT JOIN {schema}"{ self.table }" ta ON { ' AND '.join(self.joins) }"""
-
-        if distinct:
-            aliases = [col.alias for col in self.columns]
-            return f"""
-            SELECT { ', '.join(aliases) }
-            FROM (
-        { select } { ', '.join(table_columns) },
-            ROW_NUMBER() OVER(
-                PARTITION BY entities.row_id
-                { order_by }
-            ) AS row_number
-            { from_sql }
-            { wheres }
-            { order_by }
-            { group_by }
-        ) AS entities
-        WHERE row_number = 1"""
-        else:
-            return f"""
-        { select } { ', '.join(table_columns) }
-        { from_sql }
-        { wheres }
-        { order_by }
-        { group_by }"""
-
-
-@dataclass
-class FullExtractPsqlJob(FullExtractJob):
-
-    source: PostgreSQLDataSource
-    request: RetrivalRequest
-    limit: int | None = None
-
-    @property
-    def request_result(self) -> RequestResult:
-        return RequestResult.from_request(self.request)
-
-    @property
-    def retrival_requests(self) -> list[RetrivalRequest]:
-        return [self.request]
-
-    @property
-    def config(self) -> PostgreSQLConfig:
-        return self.source.config
-
-    async def to_pandas(self) -> pd.DataFrame:
-        return await self.psql_job().to_pandas()
-
-    async def to_polars(self) -> pl.LazyFrame:
-        return await self.psql_job().to_polars()
-
-    def psql_job(self) -> PostgreSqlJob:
-        return PostgreSqlJob(self.config, self.build_request())
-
-    def build_request(self) -> str:
-
-        all_features = [
-            feature.name for feature in list(self.request.all_required_features.union(self.request.entities))
-        ]
-        sql_columns = self.source.feature_identifier_for(all_features)
-        columns = [
-            f'"{sql_col}" AS {alias}' if sql_col != alias else sql_col
-            for sql_col, alias in zip(sql_columns, all_features)
-        ]
-        column_select = ', '.join(columns)
-        schema = f'{self.config.schema}.' if self.config.schema else ''
-
-        limit_query = ''
-        if self.limit:
-            limit_query = f'LIMIT {int(self.limit)}'
-
-        f'SELECT {column_select} FROM {schema}"{self.source.table}" {limit_query}',
-
-
-@dataclass
-class DateRangePsqlJob(DateRangeJob):
-
-    source: PostgreSQLDataSource
-    start_date: datetime
-    end_date: datetime
-    request: RetrivalRequest
-
-    @property
-    def request_result(self) -> RequestResult:
-        return RequestResult.from_request(self.request)
-
-    @property
-    def retrival_requests(self) -> list[RetrivalRequest]:
-        return [self.request]
-
-    @property
-    def config(self) -> PostgreSQLConfig:
-        return self.source.config
-
-    async def to_pandas(self) -> pd.DataFrame:
-        return await self.psql_job().to_pandas()
-
-    async def to_polars(self) -> pl.LazyFrame:
-        return await self.psql_job().to_polars()
-
-    def psql_job(self) -> PostgreSqlJob:
-        return PostgreSqlJob(self.config, self.build_request())
-
-    def build_request(self) -> str:
-
-        if not self.request.event_timestamp:
-            raise ValueError('Event timestamp is needed in order to run a data range job')
-
-        event_timestamp_column = self.source.feature_identifier_for([self.request.event_timestamp.name])[0]
-        all_features = [
-            feature.name for feature in list(self.request.all_required_features.union(self.request.entities))
-        ]
-        sql_columns = self.source.feature_identifier_for(all_features)
-        columns = [
-            f'"{sql_col}" AS {alias}' if sql_col != alias else sql_col
-            for sql_col, alias in zip(sql_columns, all_features)
-        ]
-        column_select = ', '.join(columns)
-        schema = f'{self.config.schema}.' if self.config.schema else ''
-        start_date = self.start_date.strftime('%Y-%m-%d %H:%M:%S')
-        end_date = self.end_date.strftime('%Y-%m-%d %H:%M:%S')
-
-        return (
-            f'SELECT {column_select} FROM {schema}"{self.source.table}" WHERE'
-            f' {event_timestamp_column} BETWEEN \'{start_date}\' AND \'{end_date}\''
-        )
-
-
 @dataclass
 class SqlValue:
     value: str | None
@@ -231,10 +41,12 @@ class FactRedshiftJob(FactualRetrivalJob):
     NB: It is expected that the data sources are for the same psql instance
     """
 
-    sources: dict[FeatureLocation, PostgreSQLDataSource]
+    sources: dict[FeatureLocation, RedshiftSQLDataSource]
     requests: list[RetrivalRequest]
     facts: RetrivalJob
 
+    entity_table_name: str = field(default='entities')
+
     @property
     def request_result(self) -> RequestResult:
         return RequestResult.from_request_list(self.requests)
@@ -244,7 +56,7 @@ class FactRedshiftJob(FactualRetrivalJob):
         return self.requests
 
     @property
-    def config(self) -> PostgreSQLConfig:
+    def config(self) -> RedshiftSQLConfig:
         return list(self.sources.values())[0].config
 
     async def to_pandas(self) -> pd.DataFrame:
@@ -257,7 +69,12 @@ class FactRedshiftJob(FactualRetrivalJob):
 
     async def psql_job(self) -> PostgreSqlJob:
         if isinstance(self.facts, PostgreSqlJob):
-            return PostgreSqlJob(self.config, self.build_sql_entity_query(self.facts))
+            return PostgreSqlJob(self.config.psql_config, self.build_sql_entity_query(self.facts))
+        raise ValueError(f'Redshift only support SQL entity queries. Got: {self.facts}')
+
+    def describe(self) -> str:
+        if isinstance(self.facts, PostgreSqlJob):
+            return PostgreSqlJob(self.config.psql_config, self.build_sql_entity_query(self.facts)).describe()
         raise ValueError(f'Redshift only support SQL entity queries. Got: {self.facts}')
 
     def dtype_to_sql_type(self, dtype: object) -> str:
@@ -277,45 +94,78 @@ class FactRedshiftJob(FactualRetrivalJob):
 
         source = self.sources[request.location]
 
-        entity_selects = {f'entities.{entity}' for entity in request.entity_names}
-        field_selects = request.all_required_feature_names.union(entity_selects).union({'entities.row_id'})
+        entity_selects = {f'{self.entity_table_name}.{entity}' for entity in request.entity_names}
+        field_selects = list(
+            request.all_required_feature_names.union(entity_selects).union(
+                {f'{self.entity_table_name}.row_id'}
+            )
+        )
         field_identifiers = source.feature_identifier_for(field_selects)
         selects = {
             SqlColumn(db_field_name, feature)
             for feature, db_field_name in zip(field_selects, field_identifiers)
+            if feature not in source.list_references
         }
 
         entities = list(request.entity_names)
         entity_db_name = source.feature_identifier_for(entities)
-        sort_query = 'entities.row_id'
+        sort_query = f'{self.entity_table_name}.row_id'
 
         event_timestamp_clause: str | None = None
         if request.event_timestamp and entities_has_event_timestamp:
             event_timestamp_column = source.feature_identifier_for([request.event_timestamp.name])[0]
-            event_timestamp_clause = f'entities.event_timestamp >= ta.{event_timestamp_column}'
+            event_timestamp_clause = (
+                f'{self.entity_table_name}.event_timestamp >= ta.{event_timestamp_column}'
+            )
             sort_query += f', {event_timestamp_column} DESC'
 
         join_conditions = [
-            f'ta."{entity_db_name}" = entities.{entity}'
+            f'ta."{entity_db_name}" = {self.entity_table_name}.{entity}'
             for entity, entity_db_name in zip(entities, entity_db_name)
         ]
         if event_timestamp_clause:
             join_conditions.append(event_timestamp_clause)
 
+        join_tables: list[tuple[TableFetch, str]] = []
+        if source.list_references:
+            for feature_name, reference in source.list_references.items():
+                if feature_name not in request.all_feature_names:
+                    continue
+                selects.add(SqlColumn(feature_name, feature_name))
+                table_id = f'{feature_name}_list'
+                source_id_column = reference.join_column or list(request.entity_names)[0]
+                column = f"'[\"' || listagg({reference.value_column}, '\",\"') || '\"]'"
+                join_table = TableFetch(
+                    name=table_id,
+                    id_column=source_id_column,
+                    table=reference.table_name,
+                    schema=reference.table_schema,
+                    columns={
+                        SqlColumn(reference.id_column, reference.id_column),
+                        SqlColumn(column, feature_name),
+                    },
+                    group_by=[reference.id_column],
+                )
+                table_join_condition = f'{table_id}.{reference.id_column} = ta.{source_id_column}'
+                join_tables.append((join_table, table_join_condition))
+
         rename_fetch = TableFetch(
             name=f'{request.name}_cte',
             id_column='row_id',
             table=source.table,
             columns=selects,
             joins=join_conditions,
+            join_tables=join_tables,
             order_by=sort_query,
-            schema=self.config.schema,
+            schema=source.config.schema,
         )
 
+        derived_map = request.derived_feature_map()
         derived_features = [
             feature
             for feature in request.derived_features
             if isinstance(feature.transformation, RedshiftTransformation)
+            and all([name not in derived_map for name in feature.depending_on_names])
         ]
         if derived_features:
             derived_alias = source.feature_identifier_for([feature.name for feature in derived_features])
@@ -353,9 +203,7 @@ class FactRedshiftJob(FactualRetrivalJob):
             time_window_config = window.window
             time_window = int(time_window_config.time_window.total_seconds())
             name = f'{request.name}_agg_{time_window}_cte'
-            group_by_names = {'entities.row_id'}
-
-        group_by_selects = {SqlColumn(feature, feature) for feature in group_by_names}
+            group_by_names = {f'{self.entity_table_name}s.row_id'}
 
         aggregates = {
             SqlColumn(
@@ -369,29 +217,33 @@ class FactRedshiftJob(FactualRetrivalJob):
         event_timestamp_clause: str | None = None
         if request.event_timestamp:
             id_column = 'row_id'
-            group_by_names = {id_column}
+            group_by_names = {f'{self.entity_table_name}.row_id'}
             # Use row_id as the main join key
             event_timestamp_name = source.feature_identifier_for([request.event_timestamp.name])[0]
             if window.window:
                 time_window_config = window.window
                 window_in_seconds = int(time_window_config.time_window.total_seconds())
                 event_timestamp_clause = (
-                    f'ta.{event_timestamp_name} BETWEEN entities.event_timestamp'
-                    f" - interval '{window_in_seconds} seconds' AND entities.event_timestamp"
+                    f'ta.{event_timestamp_name} BETWEEN {self.entity_table_name}.event_timestamp'
+                    f" - interval '{window_in_seconds} seconds' AND {self.entity_table_name}.event_timestamp"
                 )
             else:
-                event_timestamp_clause = f'ta.{event_timestamp_name} <= entities.event_timestamp'
+                event_timestamp_clause = (
+                    f'ta.{event_timestamp_name} <= {self.entity_table_name}.event_timestamp'
+                )
+
+        group_by_selects = {SqlColumn(feature, feature) for feature in group_by_names}
 
         entities = list(request.entity_names)
         entity_db_name = source.feature_identifier_for(entities)
         join_conditions = [
-            f'ta."{entity_db_name}" = entities.{entity}'
+            f'ta."{entity_db_name}" = {self.entity_table_name}.{entity}'
             for entity, entity_db_name in zip(entities, entity_db_name)
         ]
         if event_timestamp_clause:
             join_conditions.append(event_timestamp_clause)
 
-        field_selects = request.all_required_feature_names.union({'entities.*'})
+        field_selects = request.all_required_feature_names.union({f'{self.entity_table_name}.*'})
         field_identifiers = source.feature_identifier_for(field_selects)
         selects = {
             SqlColumn(db_field_name, feature)
@@ -407,17 +259,42 @@ class FactRedshiftJob(FactualRetrivalJob):
             schema=source.config.schema,
         )
 
-        needed_derived_features = set()
+        needed_derived_features: set[DerivedFeature] = set()
         derived_map = request.derived_feature_map()
 
+        def resolve_sql_transformation(feature: str) -> str:
+            """Create the sql query for hidden features.
+
+            This will loop through the hidden features,
+            and "concat" the sql query for each hidden feature.
+
+            e.g:
+            1 + x as my_feature
+            => (z + y) + x as my_feature
+            """
+            hidden_feature = derived_map[feature]
+            assert isinstance(hidden_feature.transformation, RedshiftTransformation)
+            sql_query = hidden_feature.transformation.as_redshift()
+            for depended in hidden_feature.depending_on_names:
+                if depended not in derived_map:
+                    continue
+                sub_feature = derived_map[depended]
+                if depended.isnumeric():
+                    assert isinstance(sub_feature.transformation, RedshiftTransformation)
+                    hidden_sql = sub_feature.transformation.as_redshift()
+                    hidden_sql = resolve_sql_transformation(sub_feature.name)
+                    sql_query = sql_query.replace(depended, f'({hidden_sql})')
+
+            return sql_query
+
         for agg_feature in request.aggregated_features:
             for depended in agg_feature.depending_on_names:
-                if depended in derived_map:
+                if depended in derived_map:  # if it is a derived feature
                     needed_derived_features.add(derived_map[depended])
 
         if needed_derived_features:
             features = {
-                SqlColumn(feature.transformation.as_redshift(), feature.name)
+                SqlColumn(resolve_sql_transformation(feature.name), feature.name)
                 for feature in needed_derived_features
             }.union(
                 {SqlColumn('*', '*')}
@@ -486,7 +363,7 @@ class FactRedshiftJob(FactualRetrivalJob):
 
         for request in self.requests:
             final_select_names = final_select_names.union(
-                {f'entities.{entity}' for entity in request.entity_names}
+                {f'{self.entity_table_name}.{entity}' for entity in request.entity_names}
             )
 
         if has_event_timestamp:
@@ -497,41 +374,48 @@ class FactRedshiftJob(FactualRetrivalJob):
         tables: list[TableFetch] = []
         aggregates: list[TableFetch] = []
         for request in self.requests:
-            fetch = self.value_selection(request, has_event_timestamp)
-            tables.append(fetch)
-            aggregate_fetches = self.aggregated_values_from_request(request)
-            aggregates.extend(aggregate_fetches)
 
             all_entities.update(request.entity_names)
 
-            for aggregate in aggregate_fetches:
-                agg_features = {
-                    column.alias
-                    for column in aggregate.columns
-                    if column.alias != 'entites.row_id' and column.alias not in all_entities
-                }
-                final_select_names = final_select_names.union(agg_features)
+            if request.aggregated_features:
+                aggregate_fetches = self.aggregated_values_from_request(request)
+                aggregates.extend(aggregate_fetches)
 
-            final_select_names = final_select_names.union(
-                {f'{fetch.name}.{feature}' for feature in request.all_required_feature_names}
-            )
+                for aggregate in aggregate_fetches:
+                    agg_features = {
+                        f'{aggregate.name}.{column.alias}'
+                        for column in aggregate.columns
+                        if column.alias != f'{self.entity_table_name}.row_id'
+                        and column.alias not in all_entities
+                    }
+                    final_select_names = final_select_names.union(agg_features)
+            else:
+                fetch = self.value_selection(request, has_event_timestamp)
+                tables.append(fetch)
+                final_select_names = final_select_names.union(
+                    {f'{fetch.name}.{feature}' for feature in request.all_required_feature_names}
+                )
 
         all_entities_list = list(all_entities)
         all_entities_str = ', '.join(all_entities_list)
         all_entities_list.append('row_id')
 
         entity_query = (
-            f'SELECT {all_entities_str}, ROW_NUMBER() OVER (ORDER BY '
-            f'{list(request.entity_names)[0]}) AS row_id FROM ({sql_facts.query}) AS entities'
+            f'SELECT {all_entities_str}, ROW_NUMBER() OVER ()'
+            f'AS row_id FROM ({sql_facts.query}) AS {self.entity_table_name}'
         )
         joins = '\n    '.join(
-            [f'INNER JOIN {table.name} ON {table.name}.row_id = entities.row_id' for table in tables]
+            [
+                f'LEFT JOIN {table.name} ON {table.name}.row_id = {self.entity_table_name}.row_id'
+                for table in tables
+            ]
         )
         if aggregates:
             joins += '\n    '
             joins += '\n    '.join(
                 [
-                    f'INNER JOIN {table.name} ON {table.name}.{table.id_column} = entities.{table.id_column}'
+                    f'LEFT JOIN {table.name} ON '
+                    f'{table.name}.{table.id_column} = {self.entity_table_name}.{table.id_column}'
                     for table in aggregates
                 ]
             )
@@ -558,7 +442,7 @@ class FactRedshiftJob(FactualRetrivalJob):
     ) -> str:
 
         query = f"""
-WITH entities (
+WITH {self.entity_table_name} (
     { ', '.join(entity_columns) }
 ) AS (
     { entity_query }
@@ -568,7 +452,7 @@ WITH entities (
         for table in tables:
             query += f"""
 {table.name} AS (
-    { table.sql_query(distinct='entities.row_id') }
+    { table.sql_query(distinct=f'{self.entity_table_name}.row_id') }
     ),"""
 
         # Select the aggregate features
@@ -587,19 +471,19 @@ FROM (
     SELECT { ', '.join(final_select) }
     FROM (
         SELECT *, ROW_NUMBER() OVER(
-            PARTITION BY entities.row_id
-            ORDER BY entities."{event_timestamp_col}" DESC
+            PARTITION BY {self.entity_table_name}.row_id
+            ORDER BY {self.entity_table_name}."{event_timestamp_col}" DESC
         ) AS row_number
-        FROM entities
-    ) as entities
+        FROM {self.entity_table_name}
+    ) as {self.entity_table_name}
         { final_joins }
-    WHERE entities.row_number = 1
+    WHERE {self.entity_table_name}.row_number = 1
 ) as val
 """
         else:
             query += f"""
 SELECT { ', '.join(final_select) }
-FROM entities
+FROM {self.entity_table_name}
     { final_joins }
 """
         return query
