@@ -2,19 +2,22 @@ from __future__ import annotations
 
 import asyncio
 import logging
+import timeit
 from abc import ABC, abstractmethod
 from collections import defaultdict
 from contextlib import suppress
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from datetime import datetime
 from typing import TYPE_CHECKING, TypeVar
 
 import pandas as pd
 import polars as pl
+from prometheus_client import Histogram
 
 from aligned.exceptions import UnableToFindFileException
 from aligned.request.retrival_request import RequestResult, RetrivalRequest
 from aligned.schemas.feature import Feature, FeatureType
+from aligned.schemas.vector_storage import VectorIndex
 from aligned.split_strategy import (
     SplitDataSet,
     SplitStrategy,
@@ -27,9 +30,9 @@ from aligned.validation.interface import Validator
 if TYPE_CHECKING:
     from typing import AsyncIterator
 
-    from aligned.local.source import DataFileReference
     from aligned.schemas.derivied_feature import AggregatedFeature, AggregateOver
     from aligned.schemas.model import EventTrigger
+    from aligned.sources.local import DataFileReference
 
 
 logger = logging.getLogger(__name__)
@@ -43,8 +46,7 @@ def split(
         column = data[event_timestamp_column]
         if column.dtype != 'datetime64[ns]':
             column = pd.to_datetime(data[event_timestamp_column])
-        values = column.quantile([start_ratio, end_ratio])
-        return data.loc[(column >= values.iloc[0]) & (column <= values.iloc[1])].index
+        data = data.iloc[column.sort_values().index]
 
     group_size = data.shape[0]
     start_index = round(group_size * start_ratio)
@@ -58,28 +60,32 @@ def split(
 
 
 def split_polars(
-    data: pl.LazyFrame, start_ratio: float, end_ratio: float, event_timestamp_column: str | None = None
-) -> pl.DataFrame:
-    if event_timestamp_column:
-        values = data.select(
-            [
-                pl.col(event_timestamp_column).quantile(start_ratio).alias('start_value'),
-                pl.col(event_timestamp_column).quantile(end_ratio).alias('end_value'),
-            ]
-        )
-        return data.filter(
-            pl.col(event_timestamp_column).is_between(values[0, 'start_value'], values[0, 'end_value'])
-        ).collect()
+    data: pl.DataFrame, start_ratio: float, end_ratio: float, event_timestamp_column: str | None = None
+) -> pd.Series:
 
-    collected = data.collect()
-    group_size = collected.shape[0]
+    row_name = 'row_nr'
+    data = data.with_row_count(row_name)
+
+    if event_timestamp_column:
+        data = data.sort(event_timestamp_column)
+        # values = data.select(
+        #     [
+        #         pl.col(event_timestamp_column).quantile(start_ratio).alias('start_value'),
+        #         pl.col(event_timestamp_column).quantile(end_ratio).alias('end_value'),
+        #     ]
+        # )
+        # return data.filter(
+        #     pl.col(event_timestamp_column).is_between(values[0, 'start_value'], values[0, 'end_value'])
+        # ).collect()
+
+    group_size = data.shape[0]
     start_index = round(group_size * start_ratio)
     end_index = round(group_size * end_ratio)
 
     if end_index >= group_size:
-        return collected[start_index:]
+        return data[start_index:][row_name].to_pandas()
     else:
-        return collected[start_index:end_index]
+        return data[start_index:end_index][row_name].to_pandas()
 
 
 @dataclass
@@ -122,6 +128,12 @@ class SupervisedJob:
     def with_subfeatures(self) -> SupervisedJob:
         return SupervisedJob(self.job.with_subfeatures(), self.target_columns)
 
+    def cache_raw_data(self, location: DataFileReference | str) -> SupervisedJob:
+        return SupervisedJob(
+            self.job.cache_raw_data(location),
+            self.target_columns,
+        )
+
     def cached_at(self, location: DataFileReference | str) -> SupervisedJob:
         return SupervisedJob(
             self.job.cached_at(location),
@@ -159,7 +171,7 @@ class SupervisedTrainJob:
         return TrainTestSet(
             data=data,
             entity_columns=core_data.entity_columns,
-            features=core_data.features,
+            features=core_data.feature_columns,
             target_columns=core_data.target_columns,
             train_index=split(data, 0, test_ratio_start, core_data.event_timestamp_column),
             test_index=split(data, test_ratio_start, 1, core_data.event_timestamp_column),
@@ -169,15 +181,18 @@ class SupervisedTrainJob:
     async def to_polars(self) -> TrainTestSet[pl.DataFrame]:
         # Use the pandas method, as the split is not created for polars yet
         # A but unsure if I should use the same index concept for polars
-        pandas_data = await self.to_pandas()
+        core_data = await self.job.to_polars()
+
+        data = core_data.data.collect()
+
         return TrainTestSet(
-            data=pl.from_pandas(pandas_data.data),
-            entity_columns=pandas_data.entity_columns,
-            features=pandas_data.features,
-            target_columns=pandas_data.target_columns,
-            train_index=pandas_data.train_index,
-            test_index=pandas_data.test_index,
-            event_timestamp_column=pandas_data.event_timestamp_column,
+            data=data,
+            entity_columns=core_data.entity_columns,
+            features=core_data.feature_columns,
+            target_columns=core_data.target_columns,
+            train_index=split_polars(data, 0, self.train_size, core_data.event_timestamp_column),
+            test_index=split_polars(data, self.train_size, 1, core_data.event_timestamp_column),
+            event_timestamp_column=core_data.event_timestamp_column,
         )
 
     def validation_set(self, validation_size: float) -> SupervisedValidationJob:
@@ -213,8 +228,8 @@ class SupervisedValidationJob:
         return TrainTestValidateSet(
             data=pl.from_pandas(data.data),
             entity_columns=data.entity_columns,
-            features=data.features,
-            target=data.target,
+            features=data.feature_columns,
+            target=data.labels,
             train_index=data.train_index,
             test_index=data.test_index,
             validate_index=data.validate_index,
@@ -255,7 +270,7 @@ class RetrivalJob(ABC):
 
     def log_each_job(self) -> RetrivalJob:
         if isinstance(self, ModificationJob):
-            return self.copy_with(self.job.log_each_job())
+            return LogJob(self.copy_with(self.job.log_each_job()))
         return LogJob(self)
 
     def chuncked(self, size: int) -> DataLoaderJob:
@@ -266,9 +281,14 @@ class RetrivalJob(ABC):
             return self.copy_with(self.job.with_subfeatures())
         return self
 
+    def cache_raw_data(self, location: DataFileReference | str) -> RetrivalJob:
+        if isinstance(self, ModificationJob):
+            return self.copy_with(self.job.cache_raw_data(location))
+        return self.cached_at(location)
+
     def cached_at(self, location: DataFileReference | str) -> RetrivalJob:
         if isinstance(location, str):
-            from aligned.local.source import ParquetFileSource
+            from aligned.sources.local import ParquetFileSource
 
             return FileCachedJob(ParquetFileSource(location), self)
         else:
@@ -283,8 +303,14 @@ class RetrivalJob(ABC):
     def validate(self, validator: Validator) -> RetrivalJob:
         return ValidationJob(self, validator)
 
-    def derive_features(self, requests: list[RetrivalRequest]) -> RetrivalJob:
-        return DerivedFeatureJob(job=self, requests=requests)
+    def monitor_time_used(self, time_metric: Histogram, labels: list[str] | None = None) -> RetrivalJob:
+        return TimeMetricLoggerJob(self, time_metric, labels)
+
+    def derive_features(self, requests: list[RetrivalRequest] | None = None) -> RetrivalJob:
+        return DerivedFeatureJob(job=self, requests=requests or self.retrival_requests)
+
+    def combined_features(self, requests: list[RetrivalRequest] | None = None) -> RetrivalJob:
+        return CombineFactualJob([self], requests or self.retrival_requests)
 
     def ensure_types(self, requests: list[RetrivalRequest]) -> RetrivalJob:
         return EnsureTypesJob(job=self, requests=requests)
@@ -292,21 +318,41 @@ class RetrivalJob(ABC):
     def filter(self, include_features: set[str]) -> RetrivalJob:
         return FilterJob(include_features, self)
 
+    def with_request(self, requests: list[RetrivalRequest]) -> RetrivalJob:
+        return WithRequests(self, requests)
+
     def listen_to_events(self, events: set[EventTrigger]) -> RetrivalJob:
         return ListenForTriggers(self, events)
 
+    def update_vector_index(self, indexes: list[VectorIndex]) -> RetrivalJob:
+        return UpdateVectorIndexJob(self, indexes)
+
     def validate_entites(self) -> RetrivalJob:
         return ValidateEntitiesJob(self)
 
     def fill_missing_columns(self) -> RetrivalJob:
         return FillMissingColumnsJob(self)
 
+    def rename(self, mappings: dict[str, str]) -> RetrivalJob:
+        return RenameJob(self, mappings)
+
+    def ignore_event_timestamp(self) -> RetrivalJob:
+        if isinstance(self, ModificationJob):
+            return self.copy_with(self.job.ignore_event_timestamp())
+        raise NotImplementedError('Not implemented ignore_event_timestamp')
+
     @staticmethod
     def from_dict(data: dict[str, list], request: list[RetrivalRequest] | RetrivalRequest) -> RetrivalJob:
         if isinstance(request, RetrivalRequest):
             request = [request]
         return LiteralDictJob(data, request)
 
+    @staticmethod
+    def from_polars_df(df: pl.DataFrame, request: list[RetrivalRequest]) -> RetrivalJob:
+        from aligned.local.job import LiteralRetrivalJob
+
+        return LiteralRetrivalJob(df.lazy(), RequestResult.from_request_list(request))
+
 
 JobType = TypeVar('JobType')
 
@@ -320,6 +366,58 @@ class ModificationJob:
         return self
 
 
+@dataclass
+class RenameJob(RetrivalJob, ModificationJob):
+
+    job: RetrivalJob
+    mappings: dict[str, str]
+
+    async def to_pandas(self) -> pd.DataFrame:
+        df = await self.job.to_pandas()
+        return df.rename(self.mappings)
+
+    async def to_polars(self) -> pl.LazyFrame:
+        df = await self.job.to_polars()
+        return df.rename(self.mappings)
+
+
+@dataclass
+class UpdateVectorIndexJob(RetrivalJob, ModificationJob):
+
+    job: RetrivalJob
+    indexes: list[VectorIndex]
+
+    @property
+    def request_result(self) -> RequestResult:
+        return self.job.request_result
+
+    @property
+    def retrival_requests(self) -> list[RetrivalRequest]:
+        return self.job.retrival_requests
+
+    async def to_pandas(self) -> pd.DataFrame:
+        raise NotImplementedError()
+
+    async def to_polars(self) -> pl.LazyFrame:
+        data = await self.job.to_polars()
+
+        update_jobs = []
+        for index in self.indexes:
+
+            select = index.entities + index.metadata + [index.vector]
+            select_names = {feat.name for feat in select}
+
+            filter_expr = pl.lit(True)
+            for entity in index.entities:
+                filter_expr = filter_expr & pl.col(entity.name).is_not_null()
+
+            filtered_data = data.select(select_names).filter(filter_expr)
+            update_jobs.append(index.storage.upsert_polars(filtered_data, index))
+
+        await asyncio.gather(*update_jobs)
+        return data
+
+
 @dataclass
 class LiteralDictJob(RetrivalJob):
 
@@ -340,6 +438,9 @@ class LiteralDictJob(RetrivalJob):
     async def to_polars(self) -> pl.LazyFrame:
         return pl.DataFrame(self.data).lazy()
 
+    def describe(self) -> str:
+        return f'LiteralDictJob {self.data}'
+
 
 @dataclass
 class LogJob(RetrivalJob, ModificationJob):
@@ -355,15 +456,19 @@ class LogJob(RetrivalJob, ModificationJob):
         return self.job.retrival_requests
 
     async def to_pandas(self) -> pd.DataFrame:
+        if logger.level == 0:
+            logging.basicConfig(level=logging.INFO)
         df = await self.job.to_pandas()
-        logger.info(f'Results from {type(self.job)}')
-        logger.info(df)
+        logger.debug(f'Results from {type(self.job).__name__}')
+        logger.debug(df)
         return df
 
     async def to_polars(self) -> pl.LazyFrame:
+        if logger.level == 0:
+            logging.basicConfig(level=logging.INFO)
         df = await self.job.to_polars()
-        logger.info(f'Results from {type(self.job)}')
-        logger.info(df.head(10).collect())
+        logger.debug(f'Results from {type(self.job).__name__}')
+        logger.debug(df.head(10).collect())
         return df
 
     def remove_derived_features(self) -> RetrivalJob:
@@ -389,7 +494,9 @@ class ValidationJob(RetrivalJob, ModificationJob):
 
     @property
     def features_to_validate(self) -> set[Feature]:
-        return RequestResult.from_request_list(self.retrival_requests).features
+        return RequestResult.from_request_list(
+            [request for request in self.retrival_requests if not request.aggregated_features]
+        ).features
 
     async def to_pandas(self) -> pd.DataFrame:
         return await self.validator.validate_pandas(
@@ -406,7 +513,7 @@ class ValidationJob(RetrivalJob, ModificationJob):
 
     def cached_at(self, location: DataFileReference | str) -> RetrivalJob:
         if isinstance(location, str):
-            from aligned.local.source import ParquetFileSource
+            from aligned.sources.local import ParquetFileSource
 
             return FileCachedJob(ParquetFileSource(location), self)
         else:
@@ -439,9 +546,10 @@ class DerivedFeatureJob(RetrivalJob, ModificationJob):
 
                 for feature in feature_round:
                     if feature.name in df.columns:
-                        logger.info(f'Skipped adding feature {feature.name} to computation plan')
+                        logger.debug(f'Skipped adding feature {feature.name} to computation plan')
                         continue
-                    logger.info(f'Adding feature to computation plan in polars: {feature.name}')
+
+                    logger.debug(f'Adding feature to computation plan in polars: {feature.name}')
 
                     method = await feature.transformation.transform_polars(df, feature.name)
                     if isinstance(method, pl.LazyFrame):
@@ -460,19 +568,19 @@ class DerivedFeatureJob(RetrivalJob, ModificationJob):
             for feature_round in request.derived_features_order():
                 for feature in feature_round:
                     if feature.name in df.columns:
-                        logger.info(f'Skipping to compute {feature.name} as it is aleady computed')
+                        logger.debug(f'Skipping to compute {feature.name} as it is aleady computed')
                         continue
-                    logger.info(f'Computing feature with pandas: {feature.name}')
+                    logger.debug(f'Computing feature with pandas: {feature.name}')
                     df[feature.name] = await feature.transformation.transform_pandas(
                         df[feature.depending_on_names]
                     )
-                    if df[feature.name].dtype != feature.dtype.pandas_type:
-                        if feature.dtype.is_numeric:
-                            df[feature.name] = pd.to_numeric(df[feature.name], errors='coerce').astype(
-                                feature.dtype.pandas_type
-                            )
-                        else:
-                            df[feature.name] = df[feature.name].astype(feature.dtype.pandas_type)
+                    # if df[feature.name].dtype != feature.dtype.pandas_type:
+                    #     if feature.dtype.is_numeric:
+                    #         df[feature.name] = pd.to_numeric(df[feature.name], errors='coerce').astype(
+                    #             feature.dtype.pandas_type
+                    #         )
+                    #     else:
+                    #         df[feature.name] = df[feature.name].astype(feature.dtype.pandas_type)
         return df
 
     async def to_pandas(self) -> pd.DataFrame:
@@ -522,7 +630,7 @@ class FillMissingColumnsJob(RetrivalJob, ModificationJob):
             if not missing:
                 continue
 
-            logger.info(
+            logger.warn(
                 f"""
 Some features is missing.
 Will fill values with None, but it could be a potential problem: {missing}
@@ -540,7 +648,7 @@ Will fill values with None, but it could be a potential problem: {missing}
             if not missing:
                 continue
 
-            logger.info(
+            logger.warn(
                 f"""
 Some features is missing.
 Will fill values with None, but it could be a potential problem: {missing}
@@ -579,6 +687,7 @@ class StreamAggregationJob(RetrivalJob, ModificationJob):
         if window.window:
             time_window = window.window
             filter_expr = pl.col(time_window.time_column.name) > now - time_window.time_window
+
         if window.condition:
             raise ValueError('Condition is not supported for stream aggregation, yet')
 
@@ -698,15 +807,15 @@ class RawFileCachedJob(RetrivalJob, ModificationJob):
 
     async def to_pandas(self) -> pd.DataFrame:
         from aligned.local.job import FileFullJob
-        from aligned.local.source import LiteralReference
+        from aligned.sources.local import LiteralReference
 
         try:
-            logger.info('Trying to read cache file')
+            logger.debug('Trying to read cache file')
             df = await self.location.read_pandas()
         except UnableToFindFileException:
-            logger.info('Unable to load file, so fetching from source')
+            logger.debug('Unable to load file, so fetching from source')
             df = await self.job.job.to_pandas()
-            logger.info('Writing result to cache')
+            logger.debug('Writing result to cache')
             await self.location.write_pandas(df)
         return (
             await FileFullJob(LiteralReference(df), request=self.job.requests[0])
@@ -740,23 +849,28 @@ class FileCachedJob(RetrivalJob, ModificationJob):
 
     async def to_pandas(self) -> pd.DataFrame:
         try:
-            logger.info('Trying to read cache file')
+            logger.debug('Trying to read cache file')
             df = await self.location.read_pandas()
         except UnableToFindFileException:
-            logger.info('Unable to load file, so fetching from source')
+            logger.debug('Unable to load file, so fetching from source')
             df = await self.job.to_pandas()
-            logger.info('Writing result to cache')
+            logger.debug('Writing result to cache')
             await self.location.write_pandas(df)
         return df
 
     async def to_polars(self) -> pl.LazyFrame:
         try:
-            logger.info('Trying to read cache file')
+            logger.debug('Trying to read cache file')
             df = await self.location.to_polars()
         except UnableToFindFileException:
-            logger.info('Unable to load file, so fetching from source')
+            logger.debug('Unable to load file, so fetching from source')
             df = await self.job.to_polars()
-            logger.info('Writing result to cache')
+            logger.debug('Writing result to cache')
+            await self.location.write_polars(df)
+        except FileNotFoundError:
+            logger.debug('Unable to load file, so fetching from source')
+            df = await self.job.to_polars()
+            logger.debug('Writing result to cache')
             await self.location.write_polars(df)
         return df
 
@@ -810,6 +924,59 @@ class FactualRetrivalJob(RetrivalJob):
     facts: RetrivalJob
 
 
+@dataclass
+class WithRequests(RetrivalJob, ModificationJob):
+
+    job: RetrivalJob
+    requests: list[RetrivalRequest]
+
+    @property
+    def request_result(self) -> RequestResult:
+        return RequestResult.from_request_list(self.requests)
+
+    @property
+    def retrival_requests(self) -> list[RetrivalRequest]:
+        return self.requests
+
+    async def to_pandas(self) -> pd.DataFrame:
+        return await self.job.to_pandas()
+
+    async def to_polars(self) -> pl.LazyFrame:
+        return await self.job.to_polars()
+
+
+@dataclass
+class TimeMetricLoggerJob(RetrivalJob, ModificationJob):
+
+    job: RetrivalJob
+
+    time_metric: Histogram
+    labels: list[str] | None = field(default=None)
+
+    async def to_pandas(self) -> pd.DataFrame:
+        start_time = timeit.default_timer()
+        df = await self.job.to_pandas()
+        elapsed = timeit.default_timer() - start_time
+        logger.debug(f'Computed records in {elapsed} seconds')
+        if self.labels:
+            self.time_metric.labels(*self.labels).observe(elapsed)
+        else:
+            self.time_metric.observe(elapsed)
+        return df
+
+    async def to_polars(self) -> pl.LazyFrame:
+        start_time = timeit.default_timer()
+        df = await self.job.to_polars()
+        concrete = df.collect()
+        elapsed = timeit.default_timer() - start_time
+        logger.debug(f'Computed records in {elapsed} seconds')
+        if self.labels:
+            self.time_metric.labels(*self.labels).observe(elapsed)
+        else:
+            self.time_metric.observe(elapsed)
+        return concrete.lazy()
+
+
 @dataclass
 class EnsureTypesJob(RetrivalJob, ModificationJob):
 
@@ -827,11 +994,16 @@ class EnsureTypesJob(RetrivalJob, ModificationJob):
     async def to_pandas(self) -> pd.DataFrame:
         df = await self.job.to_pandas()
         for request in self.requests:
-            for feature in request.all_required_features:
+            features_to_check = request.all_required_features
+
+            if request.aggregated_features:
+                features_to_check = {feature.derived_feature for feature in request.aggregated_features}
+
+            for feature in features_to_check:
 
                 mask = ~df[feature.name].isnull()
 
-                with suppress(AttributeError):
+                with suppress(AttributeError, TypeError):
                     df[feature.name] = df[feature.name].mask(
                         ~mask, other=df.loc[mask, feature.name].str.strip('"')
                     )
@@ -840,7 +1012,7 @@ class EnsureTypesJob(RetrivalJob, ModificationJob):
                     df[feature.name] = pd.to_datetime(df[feature.name], infer_datetime_format=True, utc=True)
                 elif feature.dtype == FeatureType('').datetime or feature.dtype == FeatureType('').string:
                     continue
-                else:
+                elif feature.dtype != FeatureType('').array:
 
                     if feature.dtype.is_numeric:
                         df[feature.name] = pd.to_numeric(df[feature.name], errors='coerce').astype(
@@ -848,7 +1020,8 @@ class EnsureTypesJob(RetrivalJob, ModificationJob):
                         )
                     else:
                         df[feature.name] = df[feature.name].astype(feature.dtype.pandas_type)
-            if request.event_timestamp:
+
+            if request.event_timestamp and request.event_timestamp.name in df.columns:
                 feature = request.event_timestamp
                 df[feature.name] = pd.to_datetime(df[feature.name], infer_datetime_format=True, utc=True)
         return df
@@ -856,21 +1029,31 @@ class EnsureTypesJob(RetrivalJob, ModificationJob):
     async def to_polars(self) -> pl.LazyFrame:
         df = await self.job.to_polars()
         for request in self.requests:
-            for feature in request.all_required_features:
+            features_to_check = request.all_required_features
+
+            if request.aggregated_features:
+                features_to_check = {feature.derived_feature for feature in request.aggregated_features}
+
+            for feature in features_to_check:
                 if feature.dtype == FeatureType('').bool:
-                    df = df.with_column(pl.col(feature.name).cast(pl.Int8).cast(pl.Boolean))
+                    df = df.with_columns(pl.col(feature.name).cast(pl.Int8).cast(pl.Boolean))
                 elif feature.dtype == FeatureType('').datetime:
                     current_dtype = df.select([feature.name]).dtypes[0]
                     if isinstance(current_dtype, pl.Datetime):
                         continue
                     # Convert from ms to us
-                    df = df.with_column(
+                    df = df.with_columns(
                         (pl.col(feature.name).cast(pl.Int64) * 1000)
                         .cast(pl.Datetime(time_zone='UTC'))
                         .alias(feature.name)
                     )
+                elif feature.dtype == FeatureType('').array:
+                    dtype = df.select(feature.name).dtypes[0]
+                    if dtype == pl.Utf8:
+                        df = df.with_columns(pl.col(feature.name).str.json_extract(pl.List(pl.Utf8)))
                 else:
-                    df = df.with_column(pl.col(feature.name).cast(feature.dtype.polars_type, strict=False))
+                    df = df.with_columns(pl.col(feature.name).cast(feature.dtype.polars_type, strict=False))
+
             if request.event_timestamp:
                 feature = request.event_timestamp
                 if feature.name not in df.columns:
@@ -878,7 +1061,7 @@ class EnsureTypesJob(RetrivalJob, ModificationJob):
                 current_dtype = df.select([feature.name]).dtypes[0]
                 if isinstance(current_dtype, pl.Datetime):
                     continue
-                df = df.with_column(
+                df = df.with_columns(
                     (pl.col(feature.name).cast(pl.Int64) * 1000)
                     .cast(pl.Datetime(time_zone='UTC'))
                     .alias(feature.name)
@@ -934,15 +1117,21 @@ class CombineFactualJob(RetrivalJob):
 
     @property
     def retrival_requests(self) -> list[RetrivalRequest]:
-        return [job.retrival_requests for job in self.jobs] + [self.combined_requests]
+        jobs = []
+        for job in self.jobs:
+            jobs.extend(job.retrival_requests)
+        return jobs + self.combined_requests
+
+    def ignore_event_timestamp(self) -> RetrivalJob:
+        return CombineFactualJob([job.ignore_event_timestamp() for job in self.jobs], self.combined_requests)
 
     async def combine_data(self, df: pd.DataFrame) -> pd.DataFrame:
         for request in self.combined_requests:
             for feature in request.derived_features:
                 if feature.name in df.columns:
-                    logger.info(f'Skipping feature {feature.name}, already computed')
+                    logger.debug(f'Skipping feature {feature.name}, already computed')
                     continue
-                logger.info(f'Computing feature: {feature.name}')
+                logger.debug(f'Computing feature: {feature.name}')
                 df[feature.name] = await feature.transformation.transform_pandas(
                     df[feature.depending_on_names]
                 )
@@ -950,11 +1139,12 @@ class CombineFactualJob(RetrivalJob):
 
     async def combine_polars_data(self, df: pl.LazyFrame) -> pl.LazyFrame:
         for request in self.combined_requests:
+            logger.debug(f'{request.name}, {len(request.derived_features)}')
             for feature in request.derived_features:
                 if feature.name in df.columns:
-                    logger.info(f'Skipping feature {feature.name}, already computed')
+                    logger.debug(f'Skipping feature {feature.name}, already computed')
                     continue
-                logger.info(f'Computing feature: {feature.name}')
+                logger.debug(f'Computing feature: {feature.name}')
                 result = await feature.transformation.transform_polars(df, feature.name)
                 if isinstance(result, pl.Expr):
                     df = df.with_columns([result.alias(feature.name)])
@@ -999,12 +1189,18 @@ class CombineFactualJob(RetrivalJob):
         # df = pl.concat(dfs_to_concat, how='horizontal')
         return await self.combine_polars_data(df)
 
+    def cache_raw_data(self, location: DataFileReference | str) -> RetrivalJob:
+        return CombineFactualJob([job.cache_raw_data(location) for job in self.jobs], self.combined_requests)
+
     def cached_at(self, location: DataFileReference | str) -> RetrivalJob:
         return CombineFactualJob([job.cached_at(location) for job in self.jobs], self.combined_requests)
 
     def remove_derived_features(self) -> RetrivalJob:
         return CombineFactualJob([job.remove_derived_features() for job in self.jobs], self.combined_requests)
 
+    def log_each_job(self) -> RetrivalJob:
+        return CombineFactualJob([job.log_each_job() for job in self.jobs], self.combined_requests)
+
     def describe(self) -> str:
         description = f'Combining {len(self.jobs)} jobs:\n'
         for index, job in enumerate(self.jobs):
@@ -1024,10 +1220,7 @@ class FilterJob(RetrivalJob, ModificationJob):
 
     @property
     def retrival_requests(self) -> list[RetrivalRequest]:
-        return [
-            request.filter_features(request.all_feature_names - self.include_features)
-            for request in self.job.retrival_requests
-        ]
+        return [request.filter_features(self.include_features) for request in self.job.retrival_requests]
 
     async def to_pandas(self) -> pd.DataFrame:
         df = await self.job.to_pandas()
@@ -1049,7 +1242,6 @@ class FilterJob(RetrivalJob, ModificationJob):
         return FilterJob(self.include_features, self.job.validate(validator))
 
     def cached_at(self, location: DataFileReference | str) -> RetrivalJob:
-
         return FilterJob(self.include_features, self.job.cached_at(location))
 
     def with_subfeatures(self) -> RetrivalJob:
@@ -1058,6 +1250,12 @@ class FilterJob(RetrivalJob, ModificationJob):
     def remove_derived_features(self) -> RetrivalJob:
         return self.job.remove_derived_features()
 
+    def ignore_event_timestamp(self) -> RetrivalJob:
+        return FilterJob(
+            include_features=self.include_features - {'event_timestamp'},
+            job=self.job.ignore_event_timestamp(),
+        )
+
 
 @dataclass
 class ListenForTriggers(RetrivalJob, ModificationJob):
