@@ -1,9 +1,10 @@
 from __future__ import annotations
 
+import asyncio
 from contextlib import suppress
 from dataclasses import dataclass, field
 from datetime import datetime
-from typing import Callable
+from typing import TYPE_CHECKING, Callable
 
 import numpy as np
 import pandas as pd
@@ -15,6 +16,9 @@ from aligned.schemas.feature import FeatureType
 from aligned.schemas.literal_value import LiteralValue
 from aligned.schemas.text_vectoriser import TextVectoriserModel
 
+if TYPE_CHECKING:
+    from aligned.sources.s3 import AwsS3Config
+
 
 @dataclass
 class TransformationTestDefinition:
@@ -118,10 +122,6 @@ class Transformation(Codable, SerializableType):
                 )
             else:
                 assert_series_equal(expected.alias(alias), output, check_names=False, check_dtype=False)
-        except pl.NotFoundError:
-            AssertionError(
-                f'Not able to find resulting transformation {cls.__name__}, remember to add .alias(alias)'
-            )
         except AttributeError:
             raise AssertionError(
                 f'Error for transformation {cls.__name__}. Could be missing a return in the transformation'
@@ -173,6 +173,7 @@ class SupportedTransformations:
             PandasFunctionTransformation,
             PolarsLambdaTransformation,
             Ratio,
+            DivideDenumeratorValue,
             Contains,
             GreaterThen,
             GreaterThenValue,
@@ -181,7 +182,10 @@ class SupportedTransformations:
             LowerThenOrEqual,
             DateComponent,
             Subtraction,
+            Multiply,
+            MultiplyValue,
             Addition,
+            AdditionValue,
             TimeDifference,
             Logarithm,
             LogarithmOnePluss,
@@ -204,8 +208,10 @@ class SupportedTransformations:
             GrayscaleImage,
             Power,
             PowerFeature,
+            PresignedAwsUrl,
             AppendConstString,
             AppendStrings,
+            PrependConstString,
             ConcatStringAggregation,
             SumAggregation,
             MeanAggregation,
@@ -217,6 +223,9 @@ class SupportedTransformations:
             StdAggregation,
             VarianceAggregation,
             PercentileAggregation,
+            JsonPath,
+            Clip,
+            ArrayContains,
         ]:
             self.add(tran_type)
 
@@ -248,20 +257,31 @@ class PandasFunctionTransformation(Transformation):
     async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
         if self.function_name not in locals():
             exec(self.code)
-        return locals()[self.function_name](df)
+
+        loaded = locals()[self.function_name]
+        if asyncio.iscoroutinefunction(loaded):
+            return await loaded(df)
+        else:
+            return loaded(df)
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
         pandas_df = df.collect().to_pandas()
         if self.function_name not in locals():
             exec(self.code)
-        pandas_df[alias] = locals()[self.function_name](pandas_df)
+
+        loaded = locals()[self.function_name]
+        if asyncio.iscoroutinefunction(loaded):
+            pandas_df[alias] = await loaded(pandas_df)
+        else:
+            pandas_df[alias] = loaded(pandas_df)
+
         return pl.from_pandas(pandas_df).lazy()
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
         return TransformationTestDefinition(
             transformation=PandasFunctionTransformation(
-                code='def test(df):\n    return df["a"] + df["b"]',
+                code='async def test(df):\n    return df["a"] + df["b"]',
                 function_name='test',
                 dtype=FeatureType('').int32,
             ),
@@ -293,7 +313,6 @@ class PandasLambdaTransformation(Transformation):
             return loaded(df)
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        import asyncio
 
         import dill
 
@@ -328,7 +347,12 @@ class PolarsFunctionTransformation(Transformation):
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
         if self.function_name not in locals():
             exec(self.code)
-        return locals()[self.function_name](df, alias)
+
+        loaded = locals()[self.function_name]
+        if asyncio.iscoroutinefunction(loaded):
+            return await loaded(df, alias)
+        else:
+            return loaded(df, alias)
 
 
 @dataclass
@@ -344,14 +368,17 @@ class PolarsLambdaTransformation(Transformation):
 
         loaded: pl.Expr = dill.loads(self.method)
         pl_df = pl.from_pandas(df)
-        pl_df = pl_df.with_column((loaded).alias('polars_tran_column'))
+        pl_df = pl_df.with_columns((loaded).alias('polars_tran_column'))
         return pl_df['polars_tran_column'].to_pandas()
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
         import dill
 
         tran: Callable[[pl.LazyFrame, str], pl.LazyFrame] = dill.loads(self.method)
-        return tran(df, alias)
+        if isinstance(tran, pl.Expr):
+            return tran
+        else:
+            return tran(df, alias)
 
 
 @dataclass
@@ -366,7 +393,7 @@ class NotNull(Transformation):
         return df[self.key].notnull()
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column(pl.col(self.key).is_not_null().alias(alias))
+        return df.with_columns(pl.col(self.key).is_not_null().alias(alias))
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
@@ -394,7 +421,7 @@ class Equals(Transformation):
         return df[self.key] == self.value.python_value
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column((pl.col(self.key) == self.value.python_value).alias(alias))
+        return pl.col(self.key) == self.value.python_value
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
@@ -426,7 +453,7 @@ class And(Transformation):
         )
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column(
+        return df.with_columns(
             (
                 pl.when(pl.col(self.first_key).is_not_null() & pl.col(self.second_key).is_not_null())
                 .then(pl.col(self.first_key) & pl.col(self.second_key))
@@ -457,7 +484,7 @@ class Or(Transformation):
         self.second_key = second_key
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column((pl.col(self.first_key) | pl.col(self.second_key)).alias(alias))
+        return df.with_columns((pl.col(self.first_key) | pl.col(self.second_key)).alias(alias))
 
     async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
         df[self.first_key].__invert__
@@ -491,11 +518,11 @@ class Inverse(Transformation):
         return gracefull_transformation(
             df,
             is_valid_mask=~(df[self.key].isnull()),
-            transformation=lambda dfv: dfv[self.key] != True,  # noqa: E712
+            transformation=lambda dfv: ~dfv[self.key].astype('bool'),
         )
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column((~pl.col(self.key)).alias(alias))
+        return df.with_columns((~pl.col(self.key)).alias(alias))
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
@@ -523,7 +550,7 @@ class NotEquals(Transformation):
         return df[self.key] != self.value.python_value
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column((pl.col(self.key) != self.value.python_value).alias(alias))
+        return pl.col(self.key) != self.value.python_value
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
@@ -547,7 +574,7 @@ class GreaterThenValue(Transformation):
         return df[self.key] > self.value
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column((pl.col(self.key) > self.value).alias(alias))
+        return pl.col(self.key) > self.value
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
@@ -573,7 +600,7 @@ class GreaterThen(Transformation):
         return df[self.left_key] > df[self.right_key]
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column((pl.col(self.left_key) > pl.col(self.right_key)).alias(alias))
+        return pl.col(self.left_key) > pl.col(self.right_key)
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
@@ -607,7 +634,7 @@ class GreaterThenOrEqual(Transformation):
         )
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column((pl.col(self.key) >= self.value).alias(alias))
+        return df.with_columns((pl.col(self.key) >= self.value).alias(alias))
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
@@ -641,7 +668,7 @@ class LowerThen(Transformation):
         )
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column((pl.col(self.key) < self.value).alias(alias))
+        return df.with_columns((pl.col(self.key) < self.value).alias(alias))
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
@@ -722,7 +749,83 @@ class Subtraction(Transformation, PsqlTransformation, RedshiftTransformation):
 
 
 @dataclass
-class Addition(Transformation):
+class AdditionValue(Transformation):
+
+    feature: str
+    value: LiteralValue
+
+    name: str = 'add_value'
+    dtype: FeatureType = FeatureType('').float
+
+    async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
+        return df[self.feature] + self.value.python_value
+
+    async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
+        return pl.col(self.feature) + pl.lit(self.value.python_value)
+
+    @staticmethod
+    def test_definition() -> TransformationTestDefinition:
+        from numpy import nan
+
+        return TransformationTestDefinition(
+            AdditionValue(feature='x', value=LiteralValue.from_value(2)),
+            input={'x': [1, 2, 0, None, 1], 'y': [1, 0, 2, 1, None]},
+            output=[3, 4, 2, nan, 3],
+        )
+
+
+@dataclass
+class Multiply(Transformation, PsqlTransformation, RedshiftTransformation):
+
+    front: str
+    behind: str
+
+    name: str = 'mul'
+    dtype: FeatureType = FeatureType('').float
+
+    def __init__(self, front: str, behind: str) -> None:
+        self.front = front
+        self.behind = behind
+
+    async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
+        return gracefull_transformation(
+            df,
+            is_valid_mask=~(df[self.front].isna() | df[self.behind].isna()),
+            transformation=lambda dfv: dfv[self.front] * dfv[self.behind],
+        )
+
+    async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
+        return pl.col(self.front) * pl.col(self.behind)
+
+    def as_psql(self) -> str:
+        return f'{self.front} * {self.behind}'
+
+
+@dataclass
+class MultiplyValue(Transformation, PsqlTransformation, RedshiftTransformation):
+
+    key: str
+    value: LiteralValue
+
+    name: str = 'mul_val'
+    dtype: FeatureType = FeatureType('').float
+
+    def __init__(self, key: str, value: LiteralValue) -> None:
+        self.key = key
+        self.value = value
+
+    async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
+        return pl.col(self.key) * pl.lit(self.value.python_value)
+
+    async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
+        return df[self.key] * self.value.python_value
+
+    def as_psql(self) -> str:
+        return f"{self.key} * '{self.value.python_value}'"
+
+
+@dataclass
+class Addition(Transformation, PsqlTransformation, RedshiftTransformation):
 
     front: str
     behind: str
@@ -744,6 +847,9 @@ class Addition(Transformation):
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
         return pl.col(self.front) + pl.col(self.behind)
 
+    def as_psql(self) -> str:
+        return f'{self.front} + {self.behind}'
+
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
         from numpy import nan
@@ -778,7 +884,7 @@ class TimeDifference(Transformation, PsqlTransformation, RedshiftTransformation)
         )
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column((pl.col(self.front) - pl.col(self.behind)).dt.seconds().alias(alias))
+        return df.with_columns((pl.col(self.front) - pl.col(self.behind)).dt.seconds().alias(alias))
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
@@ -828,7 +934,7 @@ class Logarithm(Transformation):
         )
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column(
+        return df.with_columns(
             (pl.when(pl.col(self.key) > 0).then(pl.col(self.key).log()).otherwise(pl.lit(None))).alias(alias)
         )
 
@@ -860,7 +966,7 @@ class LogarithmOnePluss(Transformation):
         )
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column(
+        return df.with_columns(
             (pl.when(pl.col(self.key) > -1).then((pl.col(self.key) + 1).log()).otherwise(pl.lit(None))).alias(
                 alias
             )
@@ -929,9 +1035,6 @@ class DateComponent(Transformation):
         )
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        pl.col(self.key).str.strptime(pl.Datetime, strict=False)
-        pl.when(pl.col(self.key))
-
         col = pl.col(self.key).cast(pl.Datetime).dt
         match self.component:
             case 'day':
@@ -1002,6 +1105,39 @@ class DateComponent(Transformation):
         )
 
 
+@dataclass
+class ArrayContains(Transformation):
+    """Checks if an array contains a value
+
+    some_array = List(String())
+    contains_a_char = some_array.contains("a")
+    """
+
+    key: str
+    value: LiteralValue
+
+    name: str = 'array_contains'
+    dtype: FeatureType = FeatureType('').bool
+
+    def __init__(self, key: str, value: str) -> None:
+        self.key = key
+        self.value = value
+
+    async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
+        return pl.Series(df[self.key]).arr.contains(self.value.python_value).to_pandas()
+
+    async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
+        return pl.col(self.key).arr.contains(self.value.python_value)
+
+    @staticmethod
+    def test_definition() -> TransformationTestDefinition:
+        return TransformationTestDefinition(
+            ArrayContains('x', LiteralValue.from_value('test')),
+            input={'x': [['Hello', 'test'], ['nah'], ['test', 'espania', None]]},
+            output=[True, False, True],
+        )
+
+
 @dataclass
 class Contains(Transformation):
     """Checks if a string value contains another string
@@ -1097,7 +1233,9 @@ class ReplaceStrings(Transformation):
         return temp_df
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        raise NotImplementedError()
+        pandas_column = df.select(self.key).collect().to_pandas()
+        transformed = await self.transform_pandas(pandas_column)
+        return df.with_columns(pl.Series(transformed).alias(alias))
 
     @staticmethod
     def test_definition() -> TransformationTestDefinition:
@@ -1151,6 +1289,41 @@ class Ratio(Transformation):
         )
 
 
+@dataclass
+class DivideDenumeratorValue(Transformation):
+
+    numerator: str
+    denumerator: LiteralValue
+
+    name: str = 'div_denum_val'
+    dtype: FeatureType = FeatureType('').float
+
+    def __init__(self, numerator: str, denumerator: LiteralValue) -> None:
+        self.numerator = numerator
+        self.denumerator = denumerator
+        assert denumerator.python_value != 0
+
+    async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
+        return gracefull_transformation(
+            df,
+            is_valid_mask=~(df[self.numerator].isna()),
+            transformation=lambda dfv: dfv[self.numerator].astype(float) / self.denumerator.python_value,
+        )
+
+    async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
+        return pl.col(self.numerator) / pl.lit(self.denumerator.python_value)
+
+    @staticmethod
+    def test_definition() -> TransformationTestDefinition:
+        from numpy import nan
+
+        return TransformationTestDefinition(
+            DivideDenumeratorValue('x', LiteralValue.from_value(2)),
+            input={'x': [1, 2, 0, 1, None, 9]},
+            output=[0.5, 1, 0, 0.5, nan, 4.5],
+        )
+
+
 @dataclass
 class IsIn(Transformation):
 
@@ -1344,7 +1517,7 @@ class MapArgMax(Transformation):
                 )
             else:
                 expr = pl.when(pl.col(key) > 0.5).then(value.python_value).otherwise(pl.lit(None))
-            return df.with_column(expr.alias(alias))
+            return df.with_columns(expr.alias(alias))
         else:
             features = list(self.column_mappings.keys())
             arg_max_alias = f'{alias}_arg_max'
@@ -1354,8 +1527,8 @@ class MapArgMax(Transformation):
                     alias: [self.column_mappings[feature].python_value for feature in features],
                     arg_max_alias: list(range(0, len(features))),
                 }
-            ).with_column(pl.col(arg_max_alias).cast(pl.UInt32))
-            sub = df.with_column(pl.concat_list(pl.col(features)).alias(array_row_alias)).with_column(
+            ).with_columns(pl.col(arg_max_alias).cast(pl.UInt32))
+            sub = df.with_columns(pl.concat_list(pl.col(features)).alias(array_row_alias)).with_columns(
                 pl.col(array_row_alias).arr.arg_max().alias(arg_max_alias)
             )
             return sub.join(mapper.lazy(), on=arg_max_alias, how='left').select(
@@ -1407,7 +1580,7 @@ class LoadImageUrl(Transformation):
         import numpy as np
         from PIL import Image
 
-        from aligned.local.source import StorageFileSource
+        from aligned.sources.local import StorageFileSource
 
         urls = df.select(self.image_url_key).collect()[self.image_url_key]
 
@@ -1479,7 +1652,7 @@ class AppendConstString(Transformation):
         return df[self.key] + self.string
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return pl.concat_str([pl.col(self.key), pl.lit(self.string)], sep='').alias(alias)
+        return pl.concat_str([pl.col(self.key).fill_null(''), pl.lit(self.string)], sep='').alias(alias)
 
 
 @dataclass
@@ -1496,11 +1669,29 @@ class AppendStrings(Transformation):
         return df[self.first_key] + self.sep + df[self.second_key]
 
     async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
-        return df.with_column(
-            pl.concat_str([pl.col(self.first_key), pl.col(self.second_key)], sep=self.sep).alias(alias)
+        return df.with_columns(
+            pl.concat_str(
+                [pl.col(self.first_key).fill_null(''), pl.col(self.second_key).fill_null('')], sep=self.sep
+            ).alias(alias)
         )
 
 
+@dataclass
+class PrependConstString(Transformation):
+
+    string: str
+    key: str
+
+    name = 'prepend_const_string'
+    dtype = FeatureType('').string
+
+    async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
+        return self.string + df[self.key]
+
+    async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
+        return pl.concat_str([pl.lit(self.string), pl.col(self.key).fill_null('')], sep='').alias(alias)
+
+
 @dataclass
 class ConcatStringAggregation(Transformation, PsqlTransformation, RedshiftTransformation):
 
@@ -1694,7 +1885,7 @@ class MedianAggregation(Transformation, PsqlTransformation, RedshiftTransformati
         return pl.col(self.key).median()
 
     def as_psql(self) -> str:
-        return f'PERCENTILE_CONT(0.5) WITHIN GROUP(ORDER BY {self.key})'
+        return f'percentile_cont(0.5) WITHIN GROUP(ORDER BY {self.key})'
 
 
 @dataclass
@@ -1714,4 +1905,82 @@ class PercentileAggregation(Transformation, PsqlTransformation, RedshiftTransfor
         return pl.col(self.key).quantile(self.percentile)
 
     def as_psql(self) -> str:
-        return f'PERCENTILE_CONT({self.percentile}) WITHIN GROUP(ORDER BY {self.key})'
+        return f'percentile_cont({self.percentile}) WITHIN GROUP(ORDER BY {self.key})'
+
+
+@dataclass
+class Clip(Transformation, PsqlTransformation, RedshiftTransformation):
+
+    key: str
+    lower: LiteralValue
+    upper: LiteralValue
+
+    name = 'clip'
+    dtype = FeatureType('').float
+
+    async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
+        return df[self.key].clip(lower=self.lower.python_value, upper=self.upper.python_value)
+
+    async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
+        return pl.col(self.key).clip(lower_bound=self.lower.python_value, upper_bound=self.upper.python_value)
+
+    def as_psql(self) -> str:
+        return (
+            f'CASE WHEN {self.key} < {self.lower} THEN {self.lower} WHEN '
+            f'{self.key} > {self.upper} THEN {self.upper} ELSE {self.key} END'
+        )
+
+    @staticmethod
+    def test_definition() -> TransformationTestDefinition:
+        return TransformationTestDefinition(
+            transformation=Clip(key='a', lower=LiteralValue.from_value(0), upper=LiteralValue.from_value(1)),
+            input={'a': [-1, 0.1, 0.9, 2]},
+            output=[0, 0.1, 0.9, 1],
+        )
+
+
+@dataclass
+class PresignedAwsUrl(Transformation):
+
+    config: AwsS3Config
+    key: str
+
+    max_age_seconds: int = field(default=30)
+
+    name = 'presigned_aws_url'
+    dtype = FeatureType('').string
+
+    async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
+        from aioaws.s3 import S3Client
+        from httpx import AsyncClient
+
+        s3 = S3Client(AsyncClient(), config=self.config.s3_config)
+        return df[self.key].apply(lambda x: s3.signed_download_url(x, max_age=self.max_age_seconds))
+
+    async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
+        from aioaws.s3 import S3Client
+        from httpx import AsyncClient
+
+        s3 = S3Client(AsyncClient(), config=self.config.s3_config)
+
+        return df.with_columns(
+            pl.col(self.key)
+            .apply(lambda x: s3.signed_download_url(x, max_age=self.max_age_seconds))
+            .alias(alias)
+        )
+
+
+@dataclass
+class JsonPath(Transformation):
+
+    key: str
+    path: str
+
+    name = 'json_path'
+    dtype = FeatureType('').string
+
+    async def transform_pandas(self, df: pd.DataFrame) -> pd.Series:
+        return pl.Series(df[self.key]).str.json_path_match(self.path).to_pandas()
+
+    async def transform_polars(self, df: pl.LazyFrame, alias: str) -> pl.LazyFrame | pl.Expr:
+        return pl.col(self.key).str.json_path_match(self.path).alias(alias)
